#!/bin/bash
#
#   Copyright (c) 2014, Heiko Schäfer <heiko@hgl.rangun.de>
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions are met:
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#       * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
#   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#   DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
#   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

header_nocache() {
  echo "Pragma-directive: no-cache";
  echo "Cache-directive: no-cache";
  echo "Cache-control: no-cache";
  echo "Pragma: no-cache";
  echo "Expires: 0";
}

header_plain() {
  echo "Content-Type: text/plain;charset=utf-8;encoding="$HTTP_ACCEPT_ENCODING
  echo
}

header_html() {
  echo "Content-Type: text/html;charset=utf-8;encoding="$HTTP_ACCEPT_ENCODING
  echo
}

export LANG="en_US.utf8"

export HGL_CONFIG_PATH=$PWD

HGL_PATH=hgl

FONTS=`fc-list :scalable=1 -f "%{family}\n" | sort | gawk -F"," '{ if($1=="Tomson Talks") { sel="selected"; } else { sel=""; } print "<option "sel" value=""\":family="$1"\">"$1"</option>"; }'| uniq`

CGIPLUG=`$HGL_PATH --list-controlsource-plugins 2>&1 | gawk -F"; " 'match($2, /CGI Control Source/) { print $2; }'`

if [ -z "$CGIPLUG" ]; then
  header_plain
  echo "Please install/enable at least the CGI Control Source Plugin to use this TEST-CGI."
  exit 1
fi

IOPLUGS=`$HGL_PATH --list-imageoutput-plugins 2>&1 | gawk -F"; " 'FNR > 1 { gsub(/^[[:space:]]+/, "", $1); print "\""$1"\""; }'`

if [[ ! ${IOPLUGS[*]} =~ "PNG" ]]; then
  header_plain
  echo "Please install/enable at least the PNG Image Output Plugin to use this TEST-CGI."
  exit 1;
fi

STYLE="<style lang=\"test/css\">* { font-family:sans-serif; }</style>"

case $QUERY_STRING in
  info)
	header_plain
	$HGL_PATH --version 2>&1
	echo
	$HGL_PATH --list-controlsource-plugins 2>&1
	echo
	$HGL_PATH --list-imageoutput-plugins 2>&1
	echo
	echo "example.hglz:"
	$HGL_PATH --quiet ./example.hglz -s 2>&1 >/dev/null
	;;
  header)
	header_html
	VERSION=`$HGL_PATH --version 2>&1 | gawk 'FNR == 1 { print $1" "$2; }'`
	cat <<EOF
<html>
	<head><title>HGL-CGI-Test</title>$STYLE</head>
	<body bgcolor="#eeeeee"">
		<h1 align="center" valign="middle">$VERSION CGI-Test (<font face="monospace" color="blue">example.hglz</font>)</h1>
	</body>
</html>
EOF
  ;;
  input)
	COLORS=`$HGL_PATH --list-color-names 2>&1 | gawk -F", " 'FNR == 2 { for(i=1; i <= NF; ++i) { print "<option value=\""$i"\">"$i"</option>"; } }'`
	
	header_nocache
	header_html
	cat <<EOF
<html>
	<head><title>HGL-CGI-Input</title>$STYLE</head>
	<body bgcolor="#eeeeee">
		<form action="$SCRIPT_NAME" method="GET" target="_pic" name="inputForm">
			<input type="hidden" name="w" value="640"><input type="hidden" name="h" value="480">
			<center>
				<table border="0">
				  <tr>
					  <td>Your name:</td><td colspan="3"><input type="text" size="15" maxlength="15" value="HGL World" name="myName"></td>
				  </tr>
				  <tr>
					  <td>Background:</td><td><select name="bgcol" onchange="document.inputForm.submit();">
					  <option>transparent</option>$COLORS</select></td>
					  <td>&nbsp;Max.&nbsp;color&nbsp;#</td><td><select name="quant" onchange="document.inputForm.submit();">
					  <option>all</option><option value="0">b/w</option><option value="1">2</option>
					  <option value="2">4</option><option value="4">16</option><option value="8">256</option></td>
				  </tr>
				  <tr>
					  <td>Font:</td><td colspan="3"><select name="font" onchange="document.inputForm.submit();">$FONTS</td>
				  </tr>
				  <tr><td align="center" colspan="4">
					  <input type="submit" value="Render it!">
					  <input type="radio" name="pproc" value="" checked onchange="document.inputForm.submit();">Normal&nbsp;
					  <input type="radio" name="pproc" value="grey" onchange="document.inputForm.submit();">Greyscale&nbsp;
					  <input type="radio" name="pproc" value="sepia" onchange="document.inputForm.submit();">Sepia&nbsp;
					  
EOF
	if [[ ${IOPLUGS[*]} =~ "CACA" ]]; then
	    echo "<input type=\"checkbox\" name=\"plug\" value=\"CACA\" onchange=\"document.inputForm.submit();\">Fancy&nbsp;SVG"
	fi
cat <<EOF
				  </td></tr>
				</table>
				<a href="$SCRIPT_NAME?info" target="_pic" title="Show information about HGL">HGL Info</a>&nbsp;
			</center>
		</form>
	</body>
</html>
EOF
	;;
  pic*|source*)
	
	WIDTH=`echo $QUERY_STRING | gawk -F\& 'match($2, /[^=]=(.*)/, m) { print m[1]; }'`
	HEIGT=`echo $QUERY_STRING | gawk -F\& 'match($3, /[^=]=(.*)/, m) { print m[1]; }'`
	BGCOL=`echo $QUERY_STRING | gawk -F\& 'match($5, /[^=]=(.*)/, m) { print m[1]; }'`
	QUANT=`echo $QUERY_STRING | gawk -F\& 'match($6, /[^=]=(.*)/, m) { print m[1]; }'`
	PPROC=`echo $QUERY_STRING | gawk -F\& 'match($8, /[^=]=(.*)/, m) { print m[1]; }'`
	FANCY=`echo $QUERY_STRING | gawk -F\& 'match($9, /[^=]=(.*)/, m) { print m[1]; }'`
	
	if [ -n "$WIDTH" ]; then
	  WIDTH="-w"$WIDTH;
	fi
	
	if [ -n "$HEIGT" ]; then
	  HEIGT="-h"$HEIGT;
	fi
	
	if [ -n "$PPROC" -a "x$PPROC" != "xsepia" ]; then
	  GREYS="--grey --auto-contrast";
	
	else
	  GREYS="";
	fi
	
	if [ -n "$PPROC" -a "x$PPROC" != "xgrey" ]; then
	  SEPIA="--sepia --auto-contrast";
	else
	  SEPIA="";
	fi
	
	if [ -n "$BGCOL" -a "x$BGCOL" != "xtransparent" ]; then
	  BGCOL="-b"$BGCOL;
	else
	  BGCOL="";
	fi

	if [ "x$FANCY" = "xCACA" ]; then
	  FANCY="-OCACA --caca-format=svg --caca-dimension=128x48";
	  ILACE="";
	else
	  FANCY="";
	  ILACE="--interlaced";
	fi
	
	if [ "x$QUANT" != "xall" -a -n "$QUANT" ]; then QUANT="-Q$QUANT"; else QUANT=""; fi

	header_nocache

	if [ -n "$FANCY" ]; then
	  echo "Content-Type: image/svg+xml;="$HTTP_ACCEPT_ENCODING
	else
	  echo "Content-Type: image/png;encoding="$HTTP_ACCEPT_ENCODING
	fi
	echo
	$HGL_PATH --quiet ./example.hglz $BGCOL $FANCY $QUANT $ILACE $WIDTH $HEIGT $GREYS $SEPIA
	;;
  *)

	header_html
	cat <<EOF
<html>
	<head><title>HGL-CGI-Input</title>$STYLE</head>
	<body style="background:url(../bg.png)">
		<center><img width="640" height="480" src="$SCRIPT_NAME?pic&$QUERY_STRING" /></center>
	</body>
</html>
EOF
	;;
esac
